/*
 Copyright (c) Sybase, Inc. 2011 All rights reserved. 
 
 In addition to the license terms set out in the Sybase License Agreement for 
 the Sybase Unwired Platform ("Program"), the following additional or different 
 rights and accompanying obligations and restrictions shall apply to the source 
 code in this file ("Code"). Sybase grants you a limited, non-exclusive, 
 non-transferable, revocable license to use, reproduce, and modify the Code 
 solely for purposes of (i) maintaining the Code as reference material to better 
 understand the operation of the Program, and (ii) development and testing of 
 applications created in connection with your licensed use of the Program. 
 The Code may not be transferred, sold, assigned, sublicensed or otherwise 
 conveyed (whether by operation of law or otherwise) to another party without 
 Sybase's prior written consent. The following provisions shall apply to any 
 modifications you make to the Code: (i) Sybase will not provide any maintenance 
 or support for modified Code or problems that result from use of modified Code; 
 (ii) Sybase expressly disclaims any warranties and conditions, express or 
 implied, relating to modified Code or any problems that result from use of the 
 modified Code; (iii) SYBASE SHALL NOT BE LIABLE FOR ANY LOSS OR DAMAGE RELATING 
 TO MODIFICATIONS MADE TO THE CODE OR FOR ANY DAMAGES RESULTING FROM USE OF THE 
 MODIFIED CODE, INCLUDING, WITHOUT LIMITATION, ANY INACCURACY OF DATA, LOSS OF 
 PROFITS OR DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, EVEN 
 IF SYBASE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES; (iv) you agree 
 to indemnify, hold harmless, and defend Sybase from and against any claims or 
 lawsuits, including attorney's fees, that arise from or are related to the 
 modified Code or from use of the modified Code.
 */

#import "HybridAppsFolderView.h"

static BOOL openAutomatically = YES;

@implementation HybridAppsFolderView

@synthesize mydelegate;
@synthesize hybridApps;
@synthesize viewController;

- (id) initFromViewController:(RootViewController*)theViewController {
   if (self = [super init]) {
      self.viewController = theViewController;
      self.hybridApps = [[HybridAppCollection alloc] init];
      self.dataSource = self;
      self.delegate = self;
      self.mydelegate = self;
      self.type = iCarouselTypeCoverFlow;
      self.bounces = YES;
      self.bounceDistance = .3;
      
      [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleNotification:) name:N_WIDGETS_FOLDER_LOADED object:hybridApps];
      [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleNotification:) name:HYBRIDAPP_CHANGED_NOTIFICATION object:nil];
   }
   return self;
}

- (void) handleNotification:(NSNotification*)notification {
#ifdef DEBUG_NOTIFICATIONS
   NSLog(@"Notification for email list: %@", [notification name]);
#endif
   
   //a possible bug there that "obj" returned is not an object of WorkflowChangeNotification class.
   //for now, just check if the notification name matches and refresh the view when UI is notified
   if ( (![[notification name] isEqualToString: N_WIDGETS_FOLDER_LOADED]) &&
        (![[notification name] isEqualToString: HYBRIDAPP_CHANGED_NOTIFICATION]) )
   {
      return;
   }

   [hybridApps loadForFolder:HybridAppFolder];
   NSLog( @"Hybrid apps count = %d (filtered = %d) after loading for folder type %d", hybridApps.count, [self getHybridAppCount], HybridAppFolder );
   
   viewController.unreadMsgCount = [HybridAppCollection unreadCount];
	[viewController displayMessageTitle];
	[self reloadData];
   
   return;
}

- (void) reloadData {
   [super reloadData];
   if (mydelegate != nil)
      [mydelegate tableViewDidReload];
}

- (void) dealloc {
   [[NSNotificationCenter defaultCenter] removeObserver:self];
   [hybridApps release];
   [viewController release];
   [super dealloc];
}

// Returns the workflow by index AFTER sorting and filtering
// Must remember to subtract 1 from index before calling to account for the "Messages" entry
- (HybridApp*) getHybridAppByIndex:(int)index {
   NSMutableArray* filteredSortedHybridApps = [[CustomizationHelper getInstance] filterAndSortApplicationPackages:hybridApps];
   if (index < filteredSortedHybridApps.count)
      return [filteredSortedHybridApps objectAtIndex:index];
   else
      return nil;
}

// Returns the number of workflows AFTER filtering
- (int) getHybridAppCount {
   return [[CustomizationHelper getInstance] filterApplicationPackages:hybridApps].count;
}

#pragma mark -
#pragma mark iCarouselDelegate

- (CGFloat)carouselItemWidth:(iCarousel *)carousel {
   return carousel.frame.size.width*7/8;
}

- (BOOL)carouselShouldWrap:(iCarousel *)carousel {
   return NO;
}

- (BOOL)carousel:(iCarousel *)carousel shouldSelectItemAtIndex:(NSInteger)index {
   int iHybridAppsAfterFiltering = [self getHybridAppCount];
   if (index < iHybridAppsAfterFiltering + 1)
      return YES;
   else
      return NO;
}

- (void)carousel:(iCarousel *)carousel didSelectItemAtIndex:(NSInteger)index {
   
   if (index == 0) {	
      WidgetFolderController *controller = [[WidgetFolderController alloc] init:HybridAppMessageFolder];
      if (nil != controller) {
         controller.hybridAppSelectionDelegate = viewController.hybridAppSelectionDelegate;
         [viewController.navigationController pushViewController:controller animated:YES];				
         [controller release];
      }
      return;
   }
   
   HybridApp *app = [self getHybridAppByIndex:index-1];
   if ( app != nil) {
      if ( UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ) {
         if ( viewController.hybridAppSelectionDelegate != nil )
            [viewController.hybridAppSelectionDelegate hybridAppSelectionChanged:app];
      }
      else {
         [viewController openHybridApp:app fromController:viewController delegate:viewController];
      }
   }
}

#pragma mark -
#pragma mark iCarouselDataSource

- (NSUInteger)numberOfItemsInCarousel:(iCarousel *)carousel {
   int iHybridAppsAfterFiltering = [self getHybridAppCount];
   
   // Special case where one cell is the "Messages" cell and the other cell is the "No Hybrid Apps" cell
   if ( iHybridAppsAfterFiltering == 0 )
      return 2;
   
   // Run this through the filtering function before determining how many to display, AND ADD 1
   return iHybridAppsAfterFiltering + 1;
}

- (UIView *)carousel:(iCarousel *)carousel viewForItemAtIndex:(NSUInteger)index reusingView:(UIView *)view {
   if ( index == 0 ) {
      return [self messagesFolderCell:carousel];
   }
   
   // Special case where one cell is the "Messages" cell and the other cell is the "No Hybrid Apps" cell
   int iHybridAppsAfterFiltering = [self getHybridAppCount];
   if ( index == 1 && iHybridAppsAfterFiltering == 0 ) {
      return [self noHybridAppsFoundCell:carousel];
   }
   
   return [self hybridAppCell:carousel forIndex:index];
}

#pragma mark -
#pragma mark SingleHybridAppTableViewDelegate

- (void) tableViewDidReload {
   if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad )
   {
      int iWorkflowsAfterFiltering = [self getHybridAppCount];
      if (iWorkflowsAfterFiltering >= 1 && openAutomatically)
      {
         openAutomatically = NO;
         HybridApp *app = [self getHybridAppByIndex:0];
         if ( app.isDefault )
         {
            [viewController openHybridApp:app fromController:viewController delegate:viewController];
         }
      }
   }
}

#pragma mark -
#pragma mark Drawing cells

- (UIView *)createCarouselViewInCarousel:(iCarousel*)carousel {
   CGRect rect = CGRectMake(0, carousel.frame.size.height/8, carousel.frame.size.width*3/4, carousel.frame.size.height/2);
   UIView* view = [[[UIView alloc] initWithFrame:rect] autorelease];
   return view;
}

- (UILabel *)createLabelInCarousel:(iCarousel*)carousel withX:(int)xVal withY:(int)yVal {
   CGRect labelFrame = CGRectMake( xVal, yVal, carousel.frame.size.width*3/4, 20 );
   UILabel* label = [[[UILabel alloc] initWithFrame: labelFrame] autorelease];
   return label;
}

- (UILabel *)createLabel:(int)xVal withY:(int)yVal withWidth:(int)width {
   CGRect labelFrame = CGRectMake( xVal, yVal, width, 20 );
   UILabel* label = [[[UILabel alloc] initWithFrame: labelFrame] autorelease];
   return label;
}

- (UIImageView *)createImageViewInCarousel:(iCarousel*)carousel withX:(int)xVal withY:(int)yVal {
   CGRect imageFrame = CGRectMake( xVal, yVal, carousel.frame.size.width*3/4, (carousel.frame.size.height/2)-20 );
   UIImageView* imageView = [[[UIImageView alloc] initWithFrame: imageFrame] autorelease];
   return imageView;
}

- (UIView*) hybridAppCell:(iCarousel*)carousel forIndex:(NSUInteger)index {
   HybridApp* app = [self getHybridAppByIndex:index-1];
   
   UIView* view = [self createCarouselViewInCarousel:carousel];
   UILabel* label = [self createLabelInCarousel:carousel withX:0 withY:carousel.frame.size.height/8];
   [label setText:app.moduleName];
   [label setTextColor:[UIColor blackColor]];
   [label setFont:[[CustomizationHelper getInstance] getHybridAppCellFont]];
   [label setTextAlignment:UITextAlignmentLeft];
   [view addSubview:label];
   
   UIImageView* imageView = [self createImageViewInCarousel:carousel withX:0 withY:(carousel.frame.size.height/8)+20];
   [imageView setImage:[[CustomizationHelper getInstance] getIconForHybridApp:app]];
   [view addSubview:imageView];
   return view;
}

- (UIView*) messagesFolderCell:(iCarousel*)carousel {
   NSString *messageTitle = NSLocalizedString( @"Messages", @"Messages" );	
   int ct = viewController.unreadMsgCount;
   if (ct > 0) {
      messageTitle = [NSString stringWithFormat:@"%@ (%d)", messageTitle, ct];
   }
   
   UIView* view = [self createCarouselViewInCarousel:carousel];
   UILabel* label = [self createLabelInCarousel:carousel withX:0 withY:carousel.frame.size.height/8];
   [label setText:messageTitle];
   [label setTextColor:[UIColor grayColor]];
   [label setFont:[[CustomizationHelper getInstance] getMessagesFolderCellFont]];
   [label setTextAlignment:UITextAlignmentCenter];
   [view addSubview:label];
   
   return view;
}

- (UIView*) noHybridAppsFoundCell:(iCarousel*)carousel {
   UIView* view = [self createCarouselViewInCarousel:carousel];
   UILabel* label = [self createLabelInCarousel:carousel withX:0 withY:carousel.frame.size.height/8];
   [label setText:NSLocalizedString( @"No HybridApps found", @"No HybridApps found" )];
   [label setTextColor:[UIColor grayColor]];
   [label setFont:[[CustomizationHelper getInstance] getNoHybridAppsFoundCellFont]];
   [label setTextAlignment:UITextAlignmentCenter];
   [view addSubview:label];
   
   return view;
}

@end
